import numpy as np
import cv2
import sys

#font for text
font = cv2.FONT_HERSHEY_SIMPLEX

e1 = cv2.getTickCount()

#selected img num for test
test_number = 20
path = "test_distance/"
#open selected image in BGR
img = cv2.imread(str(path)+"samples/test_"+str(test_number)+".jpg",1)

#copy img into grayscale
img_gray=cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

#use median blur on grayscale image
img_gray=cv2.medianBlur(img_gray,21)


#SELECT THRESHOLD

#adaptive for uneven light condition
thres = cv2.adaptiveThreshold(img_gray,255,cv2.ADAPTIVE_THRESH_MEAN_C, cv2.THRESH_BINARY_INV,121,10)

#manual threshold
#ret, thres=cv2.threshold(img_gray,88,255,cv2.THRESH_BINARY_INV)

#otsu thresholding
#ret, thres=cv2.threshold(img_gray,0,255,cv2.THRESH_BINARY_INV+cv2.THRESH_OTSU)

#show only for testing
cv2.imshow('treshold_before_morph',thres)

#rectangle kernel
kernel_size =  15
kernel = cv2.getStructuringElement(cv2.MORPH_RECT,(kernel_size, kernel_size))

#applicate morphological closing, border replicate keep border non touching objects non touching
thres = cv2.morphologyEx(thres, cv2.MORPH_CLOSE, kernel, borderType = cv2.BORDER_REPLICATE)

# Find Contours, only external and all pixels no approx.
contours, _ = cv2.findContours(thres, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE )



#for all objects(contours)
for cnt in range (0,len(contours)):

    area = cv2.contourArea(contours[cnt])
    #if object is big enough cal position
    if(area > 20):

        #cal pos. using moments
        M = cv2.moments(contours[cnt])
        cX = int(M["m10"] / M["m00"])
        cY = int(M["m01"] / M["m00"])
        print("Center: ",cX,", ",cY)

        #only for testing
        #print("Area: ",area)



#end time measurement
e2 = cv2.getTickCount()
time = (e2 - e1)/ cv2.getTickFrequency()
print("Time: ",time," s")



#draw and store results outside time measument

for cnt in range (0,len(contours)):

    area = cv2.contourArea(contours[cnt])
    #if object is big enough cal position
    if(area > 2000):

        #cal pos. using moments
        M = cv2.moments(contours[cnt])
        cX = int(M["m10"] / M["m00"])
        cY = int(M["m01"] / M["m00"])
        print("Center: ",cX,", ",cY)

        #only for testing
        #print("Area: ",area)

        #draw contour, midpoint, and print pos. time and area
        cv2.drawContours(img, contours, cnt, (0, 255, 0), 1)
        cv2.circle(img, (cX,cY), 0, (0,0,255), -1)
        cv2.putText(img, str(cX) + ", " + str(cY), (cX-25,cY-10), font, 0.8, (255, 0, 0), 1, cv2.LINE_AA)
        cv2.putText(img, "Time: " + str(time) + "s", (40,40), font, 0.8, (255, 0, 0), 1, cv2.LINE_AA)
        cv2.putText(img, "Area: " + str(area), (40,60), font, 0.8, (255, 0, 0), 1, cv2.LINE_AA)

#save result image
cv2.imwrite(path+"results/result_"+str(test_number)+".jpg",img)

#show images
#cv2.imshow('img_gray',img_gray)
cv2.imshow('treshold',thres)
cv2.imshow('kontury',img)
        
#wait for key and then exit program
cv2.waitKey(0)
cv2.destroyAllWindows()

sys.exit()
